@@ -7,6 +7,7 @@ Usage:
   liquidctl [options] set <channel> speed (<temperature> <percentage>) ...
   liquidctl [options] set <channel> speed <percentage>
   liquidctl [options] set <channel> color <mode> [<color>] ...
+  liquidctl [options] set <channel> screen <mode> [<value>]
   liquidctl --help
   liquidctl --version
 
@@ -52,7 +53,7 @@ Deprecated:
   --hid <ignored>                Ignored
 
 Copyright (C) 2018–2022  Jonas Malaco, Marshall Asch, CaseySJ, Tom Frey, Andrew
-Robertson, ParkerMc and contributors
+Robertson, ParkerMc, Aleksa Savic, Shady Nawara and contributors
 
 liquidctl incorporates work by leaty, Ksenija Stanojevic, Alexander Tong, Jens
 Neumaier, Kristóf Jakab, Sean Nelson, Chris Griffith, notaz, realies and Thomas
@@ -279,6 +280,8 @@ def _device_set_color(dev, args, **opts):
     color = map(color_from_str, args['<color>'])
     dev.set_color(args['<channel>'].lower(), args['<mode>'].lower(), color, **opts)
 
+def _device_set_screen(dev, args, **opts):
+    dev.set_screen(args["<channel>"], args["<mode>"], args["<value>"], **opts)
 
 def _device_set_speed(dev, args, **opts):
     if len(args['<temperature>']) > 0:
@@ -390,7 +393,8 @@ def main():
     log_handler.setFormatter(log_fmtter)
     logging.basicConfig(level=log_level, handlers=[log_handler])
 
-    _LOGGER.debug('liquidctl: %s', __version__)
+    _LOGGER.debug('script: %s', sys.argv[0])
+    _LOGGER.debug('version: %s', __version__)
     _LOGGER.debug('platform: %s', platform.platform())
     _log_requirements()
 
@@ -406,7 +410,6 @@ def main():
         args['--verbose'] = True
 
     opts = _make_opts(args)
-    opts['_internal_called_from_cli'] = True  # FOR INTERNAL USE ONLY, DO NOT REPLICATE ELSEWHERE
     filter_count = sum(1 for opt in opts if opt in _FILTER_OPTIONS)
     device_id = None
 
@@ -469,6 +472,8 @@ def main():
                     _device_set_speed(dev, args, **opts)
                 elif args['set'] and args['color']:
                     _device_set_color(dev, args, **opts)
+                elif args['set'] and args['screen']:
+                    _device_set_screen(dev, args, **opts)
                 else:
                     assert False, 'unreachable'
         except OSError as err:
