@@ -51,7 +51,7 @@ The subclass constructor can generally be kept unaware of the implementation
 details of the device parameter, and find_supported_devices already accepts
 keyword arguments and forwards them to the driver constructor.
 
-Copyright (C) 2019–2021  Jonas Malaco and contributors
+Copyright (C) 2019–2022  Jonas Malaco and contributors
 SPDX-License-Identifier: GPL-3.0-or-later
 """
 
@@ -59,37 +59,51 @@ import logging
 import sys
 
 import usb
+from usb.backend.openusb import USBTimeoutError
 try:
+    # The hidapi package, depending on how it's compiled, exposes one or two
+    # top level modules: hid and, optionally, hidraw.  When both are available,
+    # hid will be a libusb-based fallback implementation, and we prefer hidraw.
     import hidraw as hid
 except ModuleNotFoundError:
     import hid
+try:
+    import libusb_package
+except ModuleNotFoundError:
+    libusb_package = None
 
 from liquidctl.driver.base import BaseDriver, BaseBus, find_all_subclasses
+from liquidctl.driver.hwmon import HwmonDevice
+from liquidctl.error import Timeout
 from liquidctl.util import LazyHexRepr
 
+# Most devices respond in 1 second or (much) less; by default, only timeout if
+# something has almost surely gone wrong.
+_DEFAULT_TIMEOUT_MS = 2000
+
 _LOGGER = logging.getLogger(__name__)
 
 
 class BaseUsbDriver(BaseDriver):
     """Base driver class for generic USB devices.
 
-    Each driver should provide its own list of SUPPORTED_DEVICES, as well as
+    Each driver should provide its own list of _MATCHES, as well as
     implementations for all methods applicable to the devices is supports.
 
-    SUPPORTED_DEVICES should consist of a list of (vendor id, product
-    id, None (reserved), description, and extra kwargs) tuples.
+    _MATCHES should consist of a list of (vendor id, product id, description,
+    and extra kwargs) tuples.
 
     find_supported_devices will pass these extra kwargs, as well as any it
     receives, to the constructor.
     """
 
-    SUPPORTED_DEVICES = []
+    _MATCHES = []
 
     @classmethod
     def probe(cls, handle, vendor=None, product=None, release=None,
               serial=None, match=None, **kwargs):
         """Probe `handle` and yield corresponding driver instances."""
-        for vid, pid, _, description, devargs in cls.SUPPORTED_DEVICES:
+        for vid, pid, desc, devargs in cls._MATCHES:
             if (vendor and vendor != vid) or handle.vendor_id != vid:
                 continue
             if (product and product != pid) or handle.product_id != pid:
@@ -98,12 +112,12 @@ class BaseUsbDriver(BaseDriver):
                 continue
             if serial and handle.serial_number != serial:
                 continue
-            if match and match.lower() not in description.lower():
+            if match and match.lower() not in desc.lower():
                 continue
             consargs = devargs.copy()
             consargs.update(kwargs)
-            dev = cls(handle, description, **consargs)
-            _LOGGER.debug('instanced driver for %s', description)
+            dev = cls(handle, desc, **consargs)
+            _LOGGER.debug('%s identified: %s', cls.__name__, desc)
             yield dev
 
     def __init__(self, device, description, **kwargs):
@@ -174,7 +188,7 @@ class UsbHidDriver(BaseUsbDriver):
     def find_supported_devices(cls, **kwargs):
         """Find devices specifically compatible with this driver."""
         devs = []
-        for vid, pid, _, _, _ in cls.SUPPORTED_DEVICES:
+        for vid, pid, _, _ in cls._MATCHES:
             for dev in HidapiBus().find_devices(vendor=vid, product=pid, **kwargs):
                 if type(dev) == cls:
                     devs.append(dev)
@@ -193,6 +207,9 @@ class UsbHidDriver(BaseUsbDriver):
             assert hidinfo, 'Could not find device in HID bus'
             device = HidapiDevice(hid, hidinfo)
         super().__init__(device, description, **kwargs)
+        self._hwmon = HwmonDevice.from_hidraw(device.path)
+        if self._hwmon:
+            _LOGGER.debug('has kernel driver: %s (%s)', self._hwmon.driver, self._hwmon.path)
 
 
 class UsbDriver(BaseUsbDriver):
@@ -205,7 +222,7 @@ class UsbDriver(BaseUsbDriver):
     def find_supported_devices(cls, **kwargs):
         """Find devices specifically compatible with this driver."""
         devs = []
-        for vid, pid, _, _, _ in cls.SUPPORTED_DEVICES:
+        for vid, pid, _, _ in cls._MATCHES:
             for dev in PyUsbBus().find_devices(vendor=vid, product=pid, **kwargs):
                 if type(dev) == cls:
                     devs.append(dev)
@@ -291,21 +308,33 @@ class PyUsbDevice:
             self.usbdev.attach_kernel_driver(self.bInterfaceNumber)
             self._attached = False
 
-    def read(self, endpoint, length, timeout=None):
+    def read(self, endpoint, length, *, timeout=_DEFAULT_TIMEOUT_MS):
         """Read from endpoint."""
-        data = self.usbdev.read(endpoint, length, timeout=timeout)
+        try:
+            data = self.usbdev.read(endpoint, length, timeout=timeout)
+        except USBTimeoutError:
+            _LOGGER.debug('failed to read, timed out after %d ms', timeout)
+            raise Timeout()
         _LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
         return data
 
-    def write(self, endpoint, data, timeout=None):
+    def write(self, endpoint, data, *, timeout=_DEFAULT_TIMEOUT_MS):
         """Write to endpoint."""
         _LOGGER.debug('writting %d bytes: %r', len(data), LazyHexRepr(data))
-        return self.usbdev.write(endpoint, data, timeout=timeout)
+        try:
+            return self.usbdev.write(endpoint, data, timeout=timeout)
+        except USBTimeoutError:
+            _LOGGER.debug('write failed, timed out after %d ms', timeout)
+            raise Timeout()
 
-    def ctrl_transfer(self, *args, **kwargs):
+    def ctrl_transfer(self, *args, timeout=_DEFAULT_TIMEOUT_MS, **kwargs):
         """Submit a contrl transfer."""
         _LOGGER.debug('sending control transfer with %r, %r', args, kwargs)
-        return self.usbdev.ctrl_transfer(*args, **kwargs)
+        try:
+            return self.usbdev.ctrl_transfer(*args, **kwargs)
+        except USBTimeoutError:
+            _LOGGER.debug('control transfers failed, timed out after %d ms', timeout)
+            raise Timeout()
 
     @classmethod
     def enumerate(cls, vid=None, pid=None):
@@ -314,7 +343,12 @@ class PyUsbDevice:
             args['idVendor'] = vid
         if pid:
             args['idProduct'] = pid
-        for handle in usb.core.find(find_all=True, **args):
+        if libusb_package and (sys.platform == 'win32' or sys.platform == 'cygwin'):
+            _LOGGER.debug('using libusb_package.find')
+            find = libusb_package.find
+        else:
+            find = usb.core.find
+        for handle in find(find_all=True, **args):
             yield cls(handle)
 
     @property
@@ -403,7 +437,7 @@ class HidapiDevice:
             discarded += 1
         _LOGGER.debug('discarded %d previously enqueued reports', discarded)
 
-    def read(self, length):
+    def read(self, length, *, timeout=_DEFAULT_TIMEOUT_MS):
         """Read raw report from HID.
 
         The returned data follows the semantics of the Linux HIDRAW API.
@@ -412,9 +446,19 @@ class HidapiDevice:
         > returned data will be the report number; the report data follows,
         > beginning in the second byte. For devices which do not use numbered
         > reports, the report data will begin at the first byte.
+
+        Unlike the underlying cython-hidapi API this method wraps, pass
+        `timeout=None` to disable the default timeout.
         """
         self.hiddev.set_nonblocking(False)
-        data = self.hiddev.read(length)
+        if timeout is None:
+            timeout = 0  # cython-hidapi uses 0 for no timeout
+        elif timeout == 0:
+            timeout = 1  # smallest timeout forwarded to hid_read_timeout
+        data = self.hiddev.read(max_length=length, timeout_ms=timeout)
+        if timeout and not data:
+            _LOGGER.debug('failed to read, timed out after %d ms', timeout)
+            raise Timeout()
         _LOGGER.debug('read %d bytes: %r', len(data), LazyHexRepr(data))
         return data
 
@@ -479,6 +523,10 @@ class HidapiDevice:
         for info in infos:
             yield cls(api, info)
 
+    @property
+    def path(self):
+        return self.hidinfo['path']
+
     @property
     def vendor_id(self):
         return self.hidinfo['vendor_id']
@@ -516,10 +564,13 @@ class HidapiBus(BaseBus):
                      usb_port=None, **kwargs):
         """Find compatible HID devices."""
         handles = HidapiDevice.enumerate(hid, vendor, product)
-        drivers = sorted(find_all_subclasses(UsbHidDriver),
-                         key=lambda x: (x.__module__, x.__name__))
-        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
-                      ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbHidDriver), key=lambda x: x.__name__)
+        _LOGGER.debug('searching %s', self.__class__.__name__)
+        _LOGGER.debug(
+            '%s drivers: %s',
+            self.__class__.__name__,
+            ', '.join(map(lambda x: x.__name__, drivers))
+        )
         for handle in handles:
             if bus and handle.bus != bus:
                 continue
@@ -527,8 +578,24 @@ class HidapiBus(BaseBus):
                 continue
             if usb_port and handle.port != usb_port:
                 continue
-            _LOGGER.debug('found HID device %04x:%04x', handle.vendor_id,
-                          handle.product_id)
+            # each handle is a HIDAPI hid_device, and that can either mean one
+            # entire HID interface, or one interface ⨯ usage page ⨯ usage id
+            # product, depending on the platform and backend; but, for brevity,
+            # refer them simply as "HID devices"
+            if 'usage' in handle.hidinfo and 'usage_page' in handle.hidinfo:
+                _LOGGER.debug(
+                    'HID device: %04x:%04x (usage_page=%#06x usage=%#06x)',
+                    handle.vendor_id,
+                    handle.product_id,
+                    handle.hidinfo['usage_page'],
+                    handle.hidinfo['usage'],
+                )
+            else:
+                _LOGGER.debug(
+                    'HID device: %04x:%04x (usage n/a)',
+                    handle.vendor_id,
+                    handle.product_id,
+                )
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
 
@@ -537,10 +604,13 @@ class PyUsbBus(BaseBus):
     def find_devices(self, vendor=None, product=None, bus=None, address=None,
                      usb_port=None, **kwargs):
         """ Find compatible regular USB devices."""
-        drivers = sorted(find_all_subclasses(UsbDriver),
-                         key=lambda x: (x.__module__, x.__name__))
-        _LOGGER.debug('searching %s (%s)', self.__class__.__name__,
-                      ', '.join(map(lambda x: x.__name__, drivers)))
+        drivers = sorted(find_all_subclasses(UsbDriver), key=lambda x: x.__name__)
+        _LOGGER.debug('searching %s', self.__class__.__name__)
+        _LOGGER.debug(
+            '%s drivers: %s',
+            self.__class__.__name__,
+            ', '.join(map(lambda x: x.__name__, drivers))
+        )
         for handle in PyUsbDevice.enumerate(vendor, product):
             if bus and handle.bus != bus:
                 continue
@@ -548,7 +618,6 @@ class PyUsbBus(BaseBus):
                 continue
             if usb_port and handle.port != usb_port:
                 continue
-            _LOGGER.debug('found USB device %04x:%04x', handle.vendor_id,
-                          handle.product_id)
+            _LOGGER.debug('USB device: %04x:%04x', handle.vendor_id, handle.product_id)
             for drv in drivers:
                 yield from drv.probe(handle, vendor=vendor, product=product, **kwargs)
