@@ -2,6 +2,7 @@ import pytest
 from _testutils import MockHidapiDevice, Report, MockRuntimeStorage
 
 from liquidctl.driver.commander_pro import _quoted, _prepare_profile, _fan_mode_desc, CommanderPro
+from liquidctl.driver.hwmon import HwmonDevice
 from liquidctl.error import NotSupportedByDevice
 
 
@@ -16,19 +17,19 @@ from liquidctl.error import NotSupportedByDevice
 @pytest.fixture
 def commanderProDeviceUnconnected():
     device = MockHidapiDevice(vendor_id=0x1b1c, product_id=0x0c10, address='addr')
-    return CommanderPro(device, 'Corsair Commander Pro (experimental)', 6, 4, 2)
+    return CommanderPro(device, 'Corsair Commander Pro', 6, 4, 2)
 
 
 @pytest.fixture
 def lightingNodeProDeviceUnconnected():
     device = MockHidapiDevice(vendor_id=0x1b1c, product_id=0x0c0b, address='addr')
-    return CommanderPro(device, 'Corsair Lighting Node Pro (experimental)', 0, 0, 2)
+    return CommanderPro(device, 'Corsair Lighting Node Pro', 0, 0, 2)
 
 
 @pytest.fixture
 def commanderProDevice():
     device = MockHidapiDevice(vendor_id=0x1b1c, product_id=0x0c10, address='addr')
-    pro = CommanderPro(device, 'Corsair Commander Pro (experimental)', 6, 4, 2)
+    pro = CommanderPro(device, 'Corsair Commander Pro', 6, 4, 2)
 
 
     runtime_storage = MockRuntimeStorage(key_prefixes=['testing'])
@@ -39,21 +40,21 @@ def commanderProDevice():
 @pytest.fixture
 def lightingNodeProDevice():
     device = MockHidapiDevice(vendor_id=0x1b1c, product_id=0x0c0b, address='addr')
-    node = CommanderPro(device, 'Corsair Lighting Node Pro (experimental)', 0, 0, 2)
+    node = CommanderPro(device, 'Corsair Lighting Node Pro', 0, 0, 2)
     runtime_storage = MockRuntimeStorage(key_prefixes=['testing'])
     node.connect(runtime_storage=runtime_storage)
     return node
 
+
 @pytest.fixture
 def lightingNodeCoreDevice():
     device = MockHidapiDevice(vendor_id=0x1b1c, product_id=0x0c1a, address='addr')
-    node = CommanderPro(device, 'Corsair Lighting Node Core (experimental)', 0, 0, 1)
+    node = CommanderPro(device, 'Corsair Lighting Node Core', 0, 0, 1)
     runtime_storage = MockRuntimeStorage(key_prefixes=['testing'])
     node.connect(runtime_storage=runtime_storage)
     return node
 
 
-
 # prepare profile
 def test_prepare_profile_valid_max_rpm():
     assert _prepare_profile([[10, 400], [20, 5000]], 60) == [(10, 400), (20, 5000), (60, 5000), (60, 5000), (60, 5000), (60, 5000)]
@@ -163,34 +164,53 @@ def test_connect_lighting(lightingNodeProDeviceUnconnected):
     assert lightingNodeProDeviceUnconnected._data is not None
 
 
-def test_initialize_commander_pro(commanderProDevice):
+@pytest.mark.parametrize('has_hwmon,direct_access', [(False, False), (True, True), (True, False)])
+def test_initialize_commander_pro(commanderProDevice, has_hwmon, direct_access, tmp_path):
+    if has_hwmon and not direct_access:
+        commanderProDevice._hwmon = HwmonDevice('mock_module', tmp_path)
+        (tmp_path / 'temp1_input').write_text('10000\n')
+        (tmp_path / 'temp2_input').write_text('20000\n')
+        (tmp_path / 'temp4_input').write_text('40000\n')
+        (tmp_path / 'fan1_input').write_text('1100\n')
+        (tmp_path / 'fan1_label').write_text('fan1 3pin\n')
+        (tmp_path / 'fan2_input').write_text('1200\n')
+        (tmp_path / 'fan2_label').write_text('fan1 3pin\n')
+        (tmp_path / 'fan3_input').write_text('1300\n')
+        (tmp_path / 'fan3_label').write_text('fan1 4pin\n')
+    else:
+        if has_hwmon:
+            commanderProDevice._hwmon = HwmonDevice('invalid_module', None)
+        responses = [
+            '000009d4000000000000000000000000',  # firmware
+            '00000500000000000000000000000000',  # bootloader
+            '00010100010000000000000000000000',  # temp probes
+            '00010102000000000000000000000000'   # fan probes
+        ]
+        for d in responses:
+            commanderProDevice.device.preload_read(Report(0, bytes.fromhex(d)))
 
-    responses = [
-        '000009d4000000000000000000000000',  # firmware
-        '00000500000000000000000000000000',  # bootloader
-        '00010100010000000000000000000000',  # temp probes
-        '00010102000000000000000000000000'   # fan probes
-    ]
-    for d in responses:
-        commanderProDevice.device.preload_read(Report(0, bytes.fromhex(d)))
+    res = commanderProDevice.initialize(direct_access=direct_access)
 
-    res = commanderProDevice.initialize()
+    if has_hwmon and not direct_access:
+        assert len(res) == 10
+        i = 0
+    else:
+        assert len(res) == 12
+        assert res[0] == ('Firmware version', '0.9.212', '')
+        assert res[1] == ('Bootloader version', '0.5', '')
+        i = 2
 
-    assert len(res) == 12
-    assert res[0] == ('Firmware version', '0.9.212', '')
-    assert res[1] == ('Bootloader version', '0.5', '')
+    assert res[i + 0] == ('Temperature probe 1', True, '')
+    assert res[i + 1] == ('Temperature probe 2', True, '')
+    assert res[i + 2] == ('Temperature probe 3', False, '')
+    assert res[i + 3] == ('Temperature probe 4', True, '')
 
-    assert res[2] == ('Temperature probe 1', True, '')
-    assert res[3] == ('Temperature probe 2', True, '')
-    assert res[4] == ('Temperature probe 3', False, '')
-    assert res[5] == ('Temperature probe 4', True, '')
-
-    assert res[6] == ('Fan 1 control mode', 'DC', '')
-    assert res[7] == ('Fan 2 control mode', 'DC', '')
-    assert res[8] == ('Fan 3 control mode', 'PWM', '')
-    assert res[9] == ('Fan 4 control mode', None, '')
-    assert res[10] == ('Fan 5 control mode', None, '')
-    assert res[11] == ('Fan 6 control mode', None, '')
+    assert res[i + 4] == ('Fan 1 control mode', 'DC', '')
+    assert res[i + 5] == ('Fan 2 control mode', 'DC', '')
+    assert res[i + 6] == ('Fan 3 control mode', 'PWM', '')
+    assert res[i + 7] == ('Fan 4 control mode', None, '')
+    assert res[i + 8] == ('Fan 5 control mode', None, '')
+    assert res[i + 9] == ('Fan 6 control mode', None, '')
 
     data = commanderProDevice._data.load('fan_modes', None)
     assert data is not None
@@ -232,34 +252,51 @@ def test_initialize_lighting_node(lightingNodeProDevice):
     assert data is None
 
 
-def test_get_status_commander_pro(commanderProDevice):
-
-    responses = [
-        '000a8300000000000000000000000000',  # temp sensor 1
-        '000b6a00000000000000000000000000',  # temp sensor 2
-        '000a0e00000000000000000000000000',  # temp sensor 4
-        '0003ac00000000000000000000000000',  # fan speed 1
-        '0003ab00000000000000000000000000',  # fan speed 2
-        '0003db00000000000000000000000000',  # fan speed 3
-        '002f2200000000000000000000000000',  # get 12v
-        '00136500000000000000000000000000',  # get 5v
-        '000d1f00000000000000000000000000',  # get 3.3v
-    ]
-    for d in responses:
-        commanderProDevice.device.preload_read(Report(0, bytes.fromhex(d)))
+@pytest.mark.parametrize('has_hwmon,direct_access', [(False, False), (True, True), (True, False)])
+def test_get_status_commander_pro(commanderProDevice, has_hwmon, direct_access, tmp_path):
+    if has_hwmon and not direct_access:
+        commanderProDevice._hwmon = HwmonDevice('mock_module', tmp_path)
+        (tmp_path / 'temp1_input').write_text('26910\n')
+        (tmp_path / 'temp2_input').write_text('29220\n')
+        (tmp_path / 'temp4_input').write_text('25740\n')
+        (tmp_path / 'fan1_input').write_text('940\n')
+        (tmp_path / 'fan1_label').write_text('fan1 3pin\n')
+        (tmp_path / 'fan2_input').write_text('939\n')
+        (tmp_path / 'fan2_label').write_text('fan1 3pin\n')
+        (tmp_path / 'fan3_input').write_text('987\n')
+        (tmp_path / 'fan3_label').write_text('fan1 4pin\n')
+        (tmp_path / 'in0_input').write_text('12066\n')
+        (tmp_path / 'in1_input').write_text('4965\n')
+        (tmp_path / 'in2_input').write_text('3359\n')
+    else:
+        if has_hwmon:
+            commanderProDevice._hwmon = HwmonDevice('invalid_module', None)
+        responses = [
+            '000a8300000000000000000000000000',  # temp sensor 1
+            '000b6a00000000000000000000000000',  # temp sensor 2
+            '000a0e00000000000000000000000000',  # temp sensor 4
+            '0003ac00000000000000000000000000',  # fan speed 1
+            '0003ab00000000000000000000000000',  # fan speed 2
+            '0003db00000000000000000000000000',  # fan speed 3
+            '002f2200000000000000000000000000',  # get 12v
+            '00136500000000000000000000000000',  # get 5v
+            '000d1f00000000000000000000000000',  # get 3.3v
+        ]
+        for d in responses:
+            commanderProDevice.device.preload_read(Report(0, bytes.fromhex(d)))
 
     commanderProDevice._data.store('fan_modes', [0x01, 0x01, 0x02, 0x00, 0x00, 0x00])
     commanderProDevice._data.store('temp_sensors_connected', [0x01, 0x01, 0x00, 0x01])
 
-    res = commanderProDevice.get_status()
+    res = commanderProDevice.get_status(direct_access=direct_access)
     print(res)
 
     assert len(res) == 9
 
     # temp probes
-    assert res[0] == ('Temperature 1', 26.91, '°C')
-    assert res[1] == ('Temperature 2', 29.22, '°C')
-    assert res[2] == ('Temperature 4', 25.74, '°C')
+    assert res[0] == ('Temperature 1', pytest.approx(26.91, abs=1e-3), '°C')
+    assert res[1] == ('Temperature 2', pytest.approx(29.22, abs=1e-3), '°C')
+    assert res[2] == ('Temperature 4', pytest.approx(25.74, abs=1e-3), '°C')
 
     # fans rpm
     assert res[3] == ('Fan 1 speed', 940, 'rpm')
@@ -267,25 +304,26 @@ def test_get_status_commander_pro(commanderProDevice):
     assert res[5] == ('Fan 3 speed', 987, 'rpm')
 
     # voltages
-    assert res[6] == ('+12V rail', 12.066, 'V')
-    assert res[7] == ('+5V rail', 4.965, 'V')
-    assert res[8] == ('+3.3V rail', 3.359, 'V')
+    assert res[6] == ('+12V rail', pytest.approx(12.066, abs=1e-3), 'V')
+    assert res[7] == ('+5V rail', pytest.approx(4.965, abs=1e-3), 'V')
+    assert res[8] == ('+3.3V rail', pytest.approx(3.359, abs=1e-3), 'V')
 
-    # check the commands sent
-    sent = commanderProDevice.device.sent
-    assert len(sent) == 9
+    if not has_hwmon or direct_access:
+        # check the commands sent
+        sent = commanderProDevice.device.sent
+        assert len(sent) == 9
 
-    assert sent[0].data[0] == 0x11
-    assert sent[1].data[0] == 0x11
-    assert sent[2].data[0] == 0x11
+        assert sent[0].data[0] == 0x11
+        assert sent[1].data[0] == 0x11
+        assert sent[2].data[0] == 0x11
 
-    assert sent[3].data[0] == 0x21
-    assert sent[4].data[0] == 0x21
-    assert sent[5].data[0] == 0x21
+        assert sent[3].data[0] == 0x21
+        assert sent[4].data[0] == 0x21
+        assert sent[5].data[0] == 0x21
 
-    assert sent[6].data[0] == 0x12
-    assert sent[7].data[0] == 0x12
-    assert sent[8].data[0] == 0x12
+        assert sent[6].data[0] == 0x12
+        assert sent[7].data[0] == 0x12
+        assert sent[8].data[0] == 0x12
 
 
 def test_get_status_lighting_pro(lightingNodeProDevice):
@@ -438,7 +476,7 @@ def test_get_hw_led_channel_valid(commanderProDevice, channel, expected):
     assert res == expected
 
 
-@pytest.mark.parametrize('channel,expected', [('led', [0])])
+@pytest.mark.parametrize('channel,expected', [('led', [0]), ('any', [0])])
 def test_get_hw_led_channel_valid_node_core(lightingNodeCoreDevice, channel,
                                             expected):
     res = lightingNodeCoreDevice._get_hw_led_channels(channel)
@@ -863,8 +901,8 @@ def test_set_color_hardware_default_start_end(commanderProDevice):
     assert len(sent) == 5
 
     assert sent[3].data[0] == 0x35
-    assert sent[3].data[2] == 0x00  # start led
-    assert sent[3].data[3] == 0x01  # num leds
+    assert sent[3].data[2] == 0  # start led
+    assert sent[3].data[3] == 204  # num leds
 
     effects = commanderProDevice._data.load('saved_effects', default=None)
 
@@ -897,8 +935,7 @@ def test_set_color_hardware_start_set(commanderProDevice, startLED, expected):
 
 
 @pytest.mark.parametrize('numLED,expected', [
-    (1, 0x01), (30, 0x1e), (96, 0x60)
-    ])
+    (1, 1), (30, 30), (96, 96), (203, 203), (204, 204), (205, 204)    ])
 def test_set_color_hardware_num_leds(commanderProDevice, numLED, expected):
     ignore = Report(0, bytes(16))
     for _ in range(6):
@@ -919,7 +956,6 @@ def test_set_color_hardware_num_leds(commanderProDevice, numLED, expected):
     assert effects is not None
     assert len(effects) == 1
 
-
 def test_set_color_hardware_too_many_leds(commanderProDevice):
     ignore = Report(0, bytes(16))
     for _ in range(6):
@@ -934,7 +970,7 @@ def test_set_color_hardware_too_many_leds(commanderProDevice):
 
     assert sent[3].data[0] == 0x35
     assert sent[3].data[2] == 0xc7  # start led
-    assert sent[3].data[3] == 0x04  # num led
+    assert sent[3].data[3] == 5  # num led
 
     effects = commanderProDevice._data.load('saved_effects', default=None)
 
