@@ -1,7 +1,9 @@
 # uses the psf/black style
 
 import pytest
-from _testutils import MockHidapiDevice, Report
+import os
+
+from _testutils import MockHidapiDevice, MockPyusbDevice, Report
 
 from liquidctl.driver.hwmon import HwmonDevice
 from liquidctl.driver.kraken3 import KrakenX3, KrakenZ3
@@ -11,6 +13,8 @@ from liquidctl.driver.kraken3 import (
     _COLOR_CHANNELS_KRAKENZ,
     _SPEED_CHANNELS_KRAKENZ,
 )
+from test_krakenz3_response import krakenz3_response
+
 from liquidctl.util import HUE2_MAX_ACCESSORIES_IN_CHANNEL as MAX_ACCESSORIES
 from liquidctl.util import Hue2Accessory
 
@@ -44,7 +48,7 @@ def mock_krakenx3():
 @pytest.fixture
 def mock_krakenz3():
     raw = MockKraken(raw_led_channels=1)
-    dev = KrakenZ3(
+    dev = MockKrakenZ3(
         raw,
         "Mock Kraken Z73",
         speed_channels=_SPEED_CHANNELS_KRAKENZ,
@@ -70,10 +74,74 @@ class MockKraken(MockHidapiDevice):
             if self.raw_led_channels > 1:
                 reply[15 + 1 * MAX_ACCESSORIES] = Hue2Accessory.KRAKENX_GEN4_RING.value
                 reply[15 + 2 * MAX_ACCESSORIES] = Hue2Accessory.KRAKENX_GEN4_LOGO.value
+        elif data[0:2] == [0x30, 0x01]:
+            reply[0:2] = [0x31, 0x01]
+            reply[0x18] = 50  # lcd brightness
+            reply[0x1A] = 0  # lcd orientation
+        elif data[0:2] == [0x32, 0x1]:  # setup bucket
+            reply[14] = 0x1
+        elif data[0:2] == [0x32, 0x2]:  # delete bucker
+            reply[0:2] = [0x33, 0x02]
+            reply[14] = 0x1
+        elif data[0:2] == [0x38, 0x1]:  # switch bucket
+            reply[14] = 0x1
+
         self.preload_read(Report(0, reply))
         return super().write(data)
 
 
+class MockKrakenZ3(KrakenZ3):
+    def __init__(self, device, description, speed_channels, color_channels, **kwargs):
+        KrakenX3.__init__(self, device, description, speed_channels, color_channels, **kwargs)
+
+        self.bulk_device = MockPyusbDevice(0x1E71, 0x3008)
+        self.bulk_device.close_winusb_device = self.bulk_device.release
+
+        self.orientation = 0
+        self.brightness = 50
+
+        self.screen_mode = None
+
+    def set_screen(self, channel, mode, value, **kwargs):
+        self.screen_mode = mode
+        self.hid_data_index = 0
+        self.bulk_data_index = 0
+
+        super().set_screen(channel, mode, value, **kwargs)
+
+        assert self.hid_data_index == len(
+            krakenz3_response[self.screen_mode + "_hid"]
+        ), f"Incorrect number of hid messages sent for mode: {mode}"
+
+        if mode == "static" or mode == "gif":
+            assert (
+                self.bulk_data_index == 801
+                if mode == "static"
+                else len(krakenz3_response[self.screen_mode + "_bulk"])
+            ), f"Incorrect number of bulk messages sent for mode: {mode}"
+
+    def _write(self, data):
+        if self.screen_mode:
+            assert (
+                data == krakenz3_response[self.screen_mode + "_hid"][self.hid_data_index]
+            ), f"HID write failed, wrong data for mode: {self.screen_mode}, data index: {self.hid_data_index}"
+            self.hid_data_index += 1
+        return super()._write(data)
+
+    def _bulk_write(self, data):
+        fixed_data_index = self.bulk_data_index
+        if (
+            self.screen_mode == "static" and self.bulk_data_index > 1
+        ):  # the rest of the message should be identical to index 1
+            fixed_data_index = 1
+
+        assert (
+            data == krakenz3_response[self.screen_mode + "_bulk"][fixed_data_index]
+        ), f"Bulk write failed, wrong data for mode: {self.screen_mode}, data index: {self.bulk_data_index}"
+        self.bulk_data_index += 1
+        return super()._bulk_write(data)
+
+
 @pytest.mark.parametrize("has_hwmon,direct_access", [(False, False), (True, True), (True, False)])
 def test_krakenx3_initializes(mock_krakenx3, has_hwmon, direct_access, tmp_path):
     if has_hwmon:
@@ -142,3 +210,14 @@ def test_krakenz3_not_totally_broken(mock_krakenz3):
     _ = mock_krakenz3.get_status()
     mock_krakenz3.set_speed_profile(channel="fan", profile=iter([(20, 20), (30, 50), (40, 100)]))
     mock_krakenz3.set_fixed_speed(channel="pump", duty=50)
+
+    # set_screen should be the last set of functions called
+    mock_krakenz3.set_screen("lcd", "liquid", None)
+    mock_krakenz3.set_screen("lcd", "brightness", "60")
+    mock_krakenz3.set_screen("lcd", "orientation", "90")
+    mock_krakenz3.set_screen(
+        "lcd", "static", os.path.join(os.path.dirname(os.path.abspath(__file__)), "yellow.jpg")
+    )
+    mock_krakenz3.set_screen(
+        "lcd", "gif", os.path.join(os.path.dirname(os.path.abspath(__file__)), "rgb.gif")
+    )
